jQuery(document).ready(function () {
    countryChanged();
    // Display price check modal
    jQuery('#cp-price-check-btn').click(function () {
        if (isDynamicOptionsSelected() && !dynamicProductOptionsIsConfigured()) {
            alert('Please, configure the options. It is required for price check.');
            return false;
        }

        if (productIsBook() && !jQuery('#cp_book_count_pages').val()) {
           alert('Please, set the "Count pages". It is required for price check.');
           return false;
        }

        if (productIsPhotoPrint() && !jQuery('#cp_product_optionstype_archive_total_files').val()) {
            alert('Please, set the "Archive total files". It is required for price check.');
            return false;
        }



        tb_show('Price check', '/?TB_inline&inlineId=cp-price-check-modal&width=712');
        resetPriceCheckForm();
        displayDynamicOptions()
    });

    // Get price from price/lookup
    jQuery('#cp-price-check-submit').click(function () {
        getCheckPrice();
    });

    // Change country event
    jQuery('#cp_country').on('change', function () {
        countryChanged();
    });
});

// Get price from price/lookup
function getCheckPrice() {
    resetPriceCheckForm();
    jQuery('#cp-price-check-result').html('');
    jQuery.ajax({
        type: "POST",
        headers: {
            'Authorization': 'Bearer ' + apiKey
        },
        data: JSON.stringify(getRequestDataForPriceLookup()),
        contentType: 'application/json',
        url: 'https://api.cloudprinter.com/cloudapps/1.0/prices/lookup/',
        success: function (data) {
            jQuery('#cp-price-check-result').html('');
            jQuery('#cp-price-check-result').append('<br><b>Price: ' + data.price + ' ' + data.currency + '</b><hr>');
            jQuery('#cp-price-check-result').append('<b>Shipments:</b><br>');

            data.shipments[0].quotes.forEach(function (quote) {
                jQuery('#cp-price-check-result').append(quote.service + '('+ quote.shipping_option +'): ' + quote.price + ' ' + quote.currency + '<br>');
            });
        },
        error: function (data) {
            if (data.responseJSON.error) {
                jQuery('#cp-price-check-error').html(data.responseJSON.error.info)
            }
        },
        dataType: 'json'
    });
}

// Prepare request data
function getRequestDataForPriceLookup() {
    var data = {
        "country": jQuery('#cp_country').val(),
        "currency": currency,
        "items": [
            {
                "reference": "1",
                "product_reference": jQuery('#cp_product_reference').val(),
                "count": jQuery('.cp_count_pieces:enabled').val() || 1,
                "options": []
            }
        ]
    };

    if (jQuery('#cp_country').find(':selected').data('state-required') == '1') {
        data['state'] = jQuery('#cp_state').val()
    }


    // Set item options
    if (jQuery('.cp_product_options_type:checked').val() == 'fixed') {
        jQuery('.product_option').each(function (el) {
            var name = jQuery(this).data('name');
            if (name == 'type_archive_total_files') {
                data.items[0].options.push(
                    {
                        "option_reference": 'archive_total_files',
                        "count": jQuery(this).val()
                    }
                );
            } else {
                data.items[0].options.push(
                    {
                        "option_reference": jQuery(this).val(),
                        "count": '1'
                    }
                );
            }

        });
    } else {
        jQuery('#price-check-dynamic-options-fields select').each(function (){
            data.items[0].options.push(
                {
                    "option_reference": jQuery(this).val(),
                    "count": '1'
                }
            );
        })

        var archiveTotalFiles = jQuery('#cp_product_optionstype_archive_total_files').val();
        if (archiveTotalFiles) {
            data.items[0].options.push(
                {
                    "option_reference": 'archive_total_files',
                    "count": archiveTotalFiles
                }
            );
        }
    }

    // Set total pages for book.
    if (productIsBook()) {
        data.items[0].options.push({
            "option_reference": "total_pages",
            "count": jQuery('#cp_book_count_pages').val()
        });
    }

    return data
}

function resetPriceCheckForm() {
    jQuery('#cp-price-check-result').html('');
    jQuery('#cp-price-check-error').html('');
}

function displayDynamicOptions() {

    jQuery('#price-check-dynamic-options-fields').html('');

    if (isDynamicOptionsSelected()) {
        jQuery('#price-check-dynamic-options').show();
        jQuery('.dynamic-product_option-item').each(function (el) {
            var optionName = jQuery(this).find('.product-option-name').text();
            var options = '';
            jQuery(this).find('.dynamic-product_option-value:checked').each(function (optionEl) {
                var optionValueName = jQuery(this).data('name');
                var optionValue = jQuery(this).val();
                options += '<option value="' + optionValue + '">' + optionValueName + '</option>'
            });

            jQuery('#price-check-dynamic-options-fields').append('' +
                '<tr>' +
                '   <td>' + optionName + ':</td>' +
                '   <td><select>' + options + '</select></td>' +
                '</tr>' +
                '');
        })
    } else {
        jQuery('#price-check-dynamic-options').hide();
    }
}

function countryChanged()
{
    var countryReference = jQuery('#cp_country').val();
    if (jQuery('#cp_country').find(':selected').data('state-required') == '1') {
        jQuery('#cp_state').html("");
        jQuery.ajax({
            type: "POST",
            data: JSON.stringify({'country_reference': countryReference}),
            headers: {
                'Authorization': 'Bearer ' + apiKey
            },
            contentType: 'application/json',
            url: 'https://api.cloudprinter.com/cloudapps/1.0/shipping/states',
            success: function (data) {
                data.shipping_states.forEach(function (state) {
                    jQuery('#cp_state').append('<option value="'+ state.state_reference +'">'+ state.note +'</option>')
                });

                jQuery('#state').show();
            },
            dataType: 'json'
        });
    } else {
        jQuery('#state').hide();
    }
}

function isDynamicOptionsSelected(){
    return jQuery('.cp_product_options_type:checked').val() === 'dynamic'
}

function dynamicProductOptionsIsConfigured() {
    var success = true;
    jQuery('.dynamic-product_option-item').each(function (el) {
        if (jQuery(this).find('.dynamic-product_option-value:checked').length == 0) {
            success =  false;
        }
    });

    return success;
}
