jQuery(document).ready(function() {
    getProducts();
    getProductCategories();
    productOptionTypeChanged()

    // Select category
    jQuery('#cp_product_category').on('change', function () {
        selectedCategory = jQuery(this).val();
        jQuery('#cp_product_sub_category_field').show();
        jQuery('#product_reference_field').hide();
        jQuery('#cp_product_sub_category').html('<option value="" disabled selected hidden>Select the sub-category</option>')
        productCategories.forEach(function (category) {
            if (category.name === selectedCategory) {
                category.sub_categories.forEach(function (subCategory) {
                    jQuery('#cp_product_sub_category').append('<option value="' + subCategory.name + '">' + subCategory.name + '</option>')
                });
            }
        });

        if (selectedProductInfo) {
            jQuery('#cp_product_sub_category option[value="' + selectedProductInfo.sub_category + '"]').attr('selected', 'selected');
        }
    });

    // Select sub category
    jQuery('#cp_product_sub_category').on('change', function () {
        selectedSubCategory = jQuery(this).val();
        hideAll();

        jQuery('#cp_product_sub_category_field').show();

        // Prepare product list.
        jQuery('#cp_product_reference').append('<option value="" disabled selected hidden>Select a product</option>')
        products.forEach(function (product) {
            if (product.sub_category == selectedSubCategory) {
                jQuery('#cp_product_reference').append('<option value="' + product.product_reference + '">' + product.note + ' (' + product.availability + ' availability)</option>')
            }
        });

        // if category changed, reset the product quantity.
        if (selectedCategory != cp_product_meta.category) {
            jQuery('.cp_quantity_type:checked').attr('checked', false);
            jQuery('.count_pieces').hide();
            jQuery('.quantity_range').text('');
        }

        jQuery('#product_reference_field').show();

    });

    // Change file storage
    jQuery('.cp_file_storage').on('change', function () {
        fileStorageChanged()
    });

    // Change product option type
    jQuery('.cp_product_options_type').on('change', function () {
        productOptionTypeChanged(true)
    });

    jQuery('#cp_product_designer').on('change', function () {
        handleProductDesignerProjects(jQuery(this).val())
    });

    jQuery('.cp_file_method').on('change', function () {
        handleFileMethod()
    });

    // Select product
    jQuery('#cp_product_reference').on('change', function () {
        var selectedProduct = jQuery(this).val();
        jQuery('#cp-product-options-general').html('');
        jQuery('#cp-product-options-type').show();
        jQuery('#cp-product-config-field').show();
        jQuery('#cp-file-type').show();
        productOptionTypeChanged(true)

        // if product changed, reset quantity field
        if (selectedProduct != cp_product_meta.product_reference) {
            jQuery('.cp_quantity_type:checked').attr('checked', false);
            jQuery('.count_pieces').show();
            jQuery('.quantity_range').text('');
        }

        jQuery('#cp-price-check-btn').show();

        getProductInfo(selectedProduct, function () {
            prepareProductOptions()

            // Set fixed product option data.
            if (cp_product_meta.product_options) {
                for (var property in cp_product_meta.product_options) {
                    var isTextOptionField = ['type_archive_total_files'].indexOf(property) !== -1;

                    if (isTextOptionField) {
                        jQuery('#cp_product_options' + property).val(cp_product_meta.product_options[property]);
                    } else {
                        jQuery('#cp_product_options' + property + ' option[value="' + cp_product_meta.product_options[property] + '"]').attr('selected', 'selected');
                    }
                }
            }
        });
    });

    // Remove file
    jQuery('#local-storage-file').on('click', '.cp_remove_file', function() {
        jQuery(this).closest('.form-field').find('.file_not_uploaded').show();
        jQuery(this).closest('.form-field').find('.file_uploaded').hide();
    })
});

function hideAll() {
    jQuery('#cp_product_reference').html("");
    jQuery('#cp-price-check-btn').hide();
    jQuery('#cp-storage-type').hide();
    jQuery('#cp-file-type').hide();
    jQuery('#cp-product-config-field').hide();
    jQuery('#cp-product-options-type').hide();
    jQuery('#product_quantity').hide();
    jQuery('#product_book_count_pages').hide();
    jQuery('.book_file').hide();
    jQuery('.product_file').hide();
    jQuery('.zip_file').hide();
    jQuery('#cp-product-options-fixed').html('');
    jQuery('#cp-product-options-general').html('');
    jQuery('#cp-product-options-dynamic').hide();
    jQuery('#cp-price-check-error').html('');
}

function getProductInfo(productReference, onSuccess) {
    if (!productReference) {
        onSuccess()
        return false;
    }

    jQuery.ajax({
        type: "POST",
        data: JSON.stringify({'product_reference': productReference}),
        headers: {
            'Authorization': 'Bearer ' + apiKey
        },
        contentType: 'application/json',
        url: 'https://api.cloudprinter.com/cloudapps/1.0/products/info',
        success: function (data) {
            selectedProductInfo = data;
            setUserSelectedDunamicOptions();

            // handle quantity type
            if (selectedProductInfo.quantities.fixed && selectedProductInfo.quantities.range) {
                jQuery('#quantity_type').show();
            } else {
                jQuery('#quantity_type').hide();
            }

            jQuery('#cp_product_sub_category_field').show();
            jQuery('#cp_product_category option[value="' + selectedProductInfo.category+'"]').attr('selected','selected');
            jQuery('#cp_product_sub_category option[value="' + selectedProductInfo.sub_category+'"]').attr('selected','selected');

            if (selectedProductInfo.quantities.fixed) {
                jQuery('#single_count_pieces').hide();
                jQuery('#large_count_pieces').show();
                jQuery('#single_count_pieces input').attr('disabled', true);
                jQuery('#large_count_pieces select').attr('disabled', false);
                jQuery('#possible-quantity-values').text(selectedProductInfo.quantities.fixed.join(', '));
            }

            if (selectedProductInfo.quantities.range) {
                var allRanges = [];
                selectedProductInfo.quantities.range.forEach(function (value) {
                    allRanges.push(selectedProductInfo.quantities.range[0].min + '-' + selectedProductInfo.quantities.range[0].max)
                });
                jQuery('.quantity_range').text('(' + allRanges.join(', ') + ')');
                jQuery('#possible-quantity-values').text(allRanges.join(', '));
                jQuery('#single_count_pieces').show();
                jQuery('#large_count_pieces').hide();
                jQuery('#single_count_pieces input').attr('disabled', false);
                jQuery('#large_count_pieces select').attr('disabled', true);
            }

            handleFixedLocalFiles();

            handleFixedExternalFiles();

            setRequiredFilesTypes()

            if (productIsBook()) {
                jQuery('#product_book_count_pages').show();
            }

            if (!jQuery('#cp_file_method:checked').val()) {
                jQuery('#local-storage-file').show();
                jQuery('#cp-storage-type').show();
            }

            handleFileMethod();

            if (onSuccess) {
                onSuccess()
            }
        },
        dataType: 'json'
    });
}
function getProductCategories() {
    jQuery.ajax({
        type: "POST",
        data: {},
        headers: {
            'Authorization': 'Bearer ' + apiKey
        },
        contentType: 'application/json',
        url: 'https://api.cloudprinter.com/cloudapps/1.0/categories',
        success: function (data) {
            productCategories = data.categories;

            productCategories.forEach(function (category) {
                jQuery('#cp_product_category').append('<option value="' + category.name + '">' + category.name + '</option>')
            });




        },
        dataType: 'json'
    });
}

// Set CloudPrinter data to update product form
function setFormData() {

    getProductInfo(cp_product_meta.product_reference, function () {
        jQuery('#cp_product_category').trigger('change');
        jQuery('#cp_product_sub_category').trigger('change');
        jQuery('#cp_product_reference option[value="'+cp_product_meta.product_reference+'"]').attr('selected','selected');
        jQuery('#cp_product_reference').trigger('change');

        // Set options
        prepareProductOptions();
        if (cp_product_meta.product_options) {
            for (var property in cp_product_meta.product_options) {
                var isTextOptionField = ['type_archive_total_files'].indexOf(property) !== -1;

                if (isTextOptionField) {
                    jQuery('#cp_product_options' + property).val(cp_product_meta.product_options[property]);
                } else {
                    jQuery('#cp_product_options' + property + ' option[value="' + cp_product_meta.product_options[property] + '"]').attr('selected', 'selected');
                }
            }
        }

        if (cp_product_meta.book_count_pages) {
            jQuery('#cp_book_count_pages').val(cp_product_meta.book_count_pages);
        }


        handleFileMethod();
        handleProductDesignerProjects(jQuery('#cp_product_designer').val());

        // Set fixed product option data.
        if (cp_product_meta.product_options) {
            for (var property in cp_product_meta.product_options) {
                var isTextOptionField = ['type_archive_total_files'].indexOf(property) !== -1;

                if (isTextOptionField) {
                    jQuery('#cp_product_options' + property).val(cp_product_meta.product_options[property]);
                } else {
                    jQuery('#cp_product_options' + property + ' option[value="' + cp_product_meta.product_options[property] + '"]').attr('selected', 'selected');
                }
            }
        }

        // Set dynamic product option data.
        setUserSelectedDunamicOptions();

        // order quantity
        if (cp_product_meta.count_pieces) {
            jQuery('.cp_count_pieces').val(cp_product_meta.count_pieces)
        }

        switch (cp_product_meta.quantity_type) {
            case 'single':
                jQuery('#single_count_pieces').show();
                jQuery('#large_count_pieces').hide();
                jQuery('#single_count_pieces input').attr('disabled', false);
                jQuery('#large_count_pieces select').attr('disabled', true);
                break;
            case 'large':
                jQuery('#single_count_pieces').hide();
                jQuery('#large_count_pieces').show();
                jQuery('#single_count_pieces input').attr('disabled', true);
                jQuery('#large_count_pieces select').attr('disabled', false);
                break;
            default:
                switch (jQuery('.cp_quantity_type:enabled').length) {
                    case 1:
                        jQuery('.cp_quantity_type:enabled').attr('checked', true).trigger("change");
                        break;
                    case 2:
                        jQuery('#cp_quantity_type_single').attr('checked', true).trigger("change");
                        break;
                }
        }
    })
}

// Quantity type is changing.
jQuery('.cp_quantity_type').on('change', function () {
    switch (jQuery(this).val()) {
        case 'single':
            jQuery('#single_count_pieces').show();
            jQuery('#large_count_pieces').hide();
            jQuery('#single_count_pieces input').attr('disabled', false);
            jQuery('#large_count_pieces select').attr('disabled', true);
            break;
        case 'large':
            jQuery('#single_count_pieces').hide();
            jQuery('#large_count_pieces').show();
            jQuery('#single_count_pieces input').attr('disabled', true);
            jQuery('#large_count_pieces select').attr('disabled', false);
            break;
    }
});

// Display product options after select product
function prepareProductOptions() {
    if (!selectedProductInfo || !selectedProductInfo.options) {
        return;
    }

    var productOptions = {};
    jQuery('#cp-product-options-fixed').html('<hr>');

    selectedProductInfo.options.forEach(function (option) {
        if (!productOptions[option.type]) {
            productOptions[option.type] = option.type_name;
        }
    });

    jQuery('.count_pieces_warning').text('');

    // product quantity
    if (selectedProductInfo.quantities.fixed) {
        jQuery('#cp_quantity_type_large').attr('disabled', false);
        jQuery('#cp_large_count_pieces').html("");
        selectedProductInfo.quantities.fixed.forEach(function (value) {
            jQuery('#cp_large_count_pieces').append("<option value='" + value + "'>" + value + "</option>");
        });
    } else {
        jQuery('#cp_quantity_type_large').attr('disabled', true);
    }

    if (selectedProductInfo.quantities.range) {
        jQuery('#cp_quantity_type_single').attr('disabled', false);
    } else {
        jQuery('#cp_quantity_type_single').attr('disabled', true);
    }

    if (selectedProductInfo.quantities.range && selectedProductInfo.quantities.fixed)  {
        jQuery('#cp_quantity_type_single').attr('disabled', false);
        jQuery('#cp_quantity_type_large').attr('disabled', false);
    }

    if (selectedProductInfo.specs.min_order_quantity) {
        var countPieces = parseInt(jQuery('#cp_count_pieces').val());
        var minOrderQuantity = parseInt(selectedProductInfo.specs.min_order_quantity.value);
        if (countPieces < minOrderQuantity) {
            jQuery('.count_pieces_warning').text('Min count pieces should be ' + minOrderQuantity);
        }
    }

    jQuery('#product_quantity').show();
    jQuery('#cp-product-options-dynamic-content').html("");
    for (var property in productOptions) {
        var isTextOptionField = ['type_archive_total_files'].indexOf(property) !== -1;

        if (isTextOptionField) {
            jQuery('#cp-product-options-general').append(
                '<p id="product_reference_field" class="form-field cp_product_reference_field" style="position: relative; float: left;">' +
                '    <label for="cp_product_options' + property + '">' + productOptions[property] + '</label>' +
                '    <input type="number" data-name="' + property + '" id="cp_product_options' + property + '" name="cp_product_options[' + property + ']" class="select short product_option"></input>' +
                '</p>'
            )
        } else {
            // Render the fixed product options
            jQuery('#cp-product-options-fixed').append(
                '<p id="product_reference_field" class="form-field cp_product_reference_field">' +
                '    <label for="cp_product_options' + property + '">' + productOptions[property] + '</label>' +
                '    <select style="" id="cp_product_options' + property + '" name="cp_product_options[' + property + ']" class="select short product_option"></select>' +
                '</p>'
            );

            selectedProductInfo.options.forEach(function (option) {
                if (option.type === property) {
                    var optionName = option.note + '(' + option.availability + ' availability)';

                    if (option.default) {
                        optionName += '(Standard)'
                    }

                    jQuery('#cp_product_options' + property).append('<option value="' + option.option_reference + '">' + optionName+ '</option>')
                }
            });

            // Render the dynamic product options
            jQuery('#cp-product-options-dynamic-content').append(
                '<div class="dynamic-product_option-item">' +
                '   <b class="product-option-name">' + productOptions[property] + '</b>' +
                '   <div class="cp-product-config-field">' +
                '       <p class="cp-space-reset">Option title</p>' +
                '       <input ' +
                '           id="cp_dynamic_product_option_name_' + property + '"' +
                '           type="text" ' +
                '           value="' + productOptions[property] + '" ' +
                '           name="cp_dynamic_product_options[' + property + '][name]">' +
                '   </div>' +
                '   <div class="cp-product-config-field">' +
                '      <table data-option-key="' + property + '" class="widefat striped" id="cp_product_options_dynamic' + property +'" style="width: 100%">' +
                '          <thead>' +
                '              <tr>' +
                '                  <td width="50%">Option values</td>' +
                '                  <td>Options values title</td>' +
                '                  <td>Additional price (' + currencySymbol + ')</td>' +
                '              </tr>' +
                '          </thead>' +
                '      </table>' +
                '   </div>' +
                '</div>' +
                '<hr style="width: 100%;">'
            );

            selectedProductInfo.options.forEach(function (option) {
                if (option.type === property) {
                    var optionName = option.note + '(' + option.availability + ' availability)';

                    if (option.default) {
                        optionName += '(Standard)'
                    }
                    jQuery('#cp_product_options_dynamic' + property).append('' +
                        '<tr>' +
                        '   <td>' +
                        '       <input ' +
                        '           id="cp_dynamic_product_option_value_' + option.option_reference + '" ' +
                        '           data-name="' + optionName + '"' +
                        '           class="dynamic-product_option-value"' +
                        '           name="cp_dynamic_product_options[' + property + '][values][]"' +
                        '           value="' + option.option_reference + '" ' +
                        '           type="checkbox"> ' +
                        '       '+ optionName +'' +
                        '   </td>' +
                        '   <td>' +
                        '       <input ' +
                        '           id="cp_dynamic_product_option_value_name_' + option.option_reference + '"' +
                        '           type="text" ' +
                        '           value="' + option.note + '"' +
                        '           name="cp_dynamic_product_options[' + property + '][names][' + option.option_reference + ']">' +
                        '   </td>' +
                        '   <td>' +
                        '       <input ' +
                        '           id="cp_dynamic_product_option_price_' + option.option_reference + '"' +
                        '           type="number" ' +
                        '           step="any"' +
                        '           value="0"' +
                        '           name="cp_dynamic_product_options[' + property + '][prices][' + option.option_reference + ']">' +
                        '   </td>' +
                        '</tr>'
                    )
                }
            });
            setUserSelectedDunamicOptions();
        }
    }
}

// Get product list.
function getProducts() {
    jQuery.ajax({
        type: "POST",
        headers: {
            'Authorization': 'Bearer ' + apiKey
        },
        contentType: 'application/json',
        url: 'https://api.cloudprinter.com/cloudapps/1.0/products',
        success: function (data) {
            products = data;
            setFormData();
        },
        dataType: 'json'
    });
}

function productIsBook() {
    return ['Textbook BW', 'Textbook FC', 'Photobook', 'Brochure', 'Magazine'].indexOf(jQuery('#cp_product_category').val()) !== -1;
}

function productIsPhotoPrint() {
    return 'Photo print' === jQuery('#cp_product_category').val()
}

// Check if file is required for specific type.
function fileIsRequired(type, productInfo) {
    for (file of productInfo.files) {
        if (file.type == type) {
            return true;
        }
    }

    return false;
}

function setRequiredFilesTypes() {
    if (!selectedProductInfo) {
        return false;
    }
    var requiredFilesTypes = [];
    for (file of selectedProductInfo.files) {
        requiredFilesTypes.push(file.type);
    }

    jQuery('#cp_product_file_required').val(requiredFilesTypes.join('|'));
}

function handleFixedExternalFiles() {
    jQuery('#external-storage-file').html("");

    if (!selectedProductInfo) {
        return false;
    }

    for (file of selectedProductInfo.files) {

        if (
            cp_product_meta.productFiles.external !== undefined
            && cp_product_meta.productFiles.external[file.type] !== undefined
        ) {
            var md5 = cp_product_meta.productFiles.external[file.type].md5;
            var link = cp_product_meta.productFiles.external[file.type].link;
        } else {
            var md5 = '', link = ''

        }

        jQuery('#external-storage-file').append('' +
            '<p class="form-field product_file">' +
            '   <label for="cp_file_url_' + file.type + '">' +
                    file.type[0].toUpperCase() + file.type.substring(1) + ' file (' + file.format +')' +
            '   </label>' +
            '   <input id="cp_file_url_' + file.type + '" type="url" name="cp_file_url[' + file.type + ']" class="cp-input" value="' + link + '">' +
            '</p>' +
            '<p class="form-field ' + file.type + '_file">' +
            '    <label for="cp_file_url_' + file.type + '_md5">' +
                    file.type[0].toUpperCase() + file.type.substring(1) + ' md5 hash' +
            '    </label>' +
            '    <input id="cp_file_url_' + file.type + '_md5" name="cp_file_url_md5[' + file.type + ']" class="cp-input" value="' + md5 + '">' +
            '</p>'
        );
    }
}

function handleFixedLocalFiles() {
    jQuery('#local-storage-file').html("");

    if (!selectedProductInfo) {
        return false;
    }

    for (file of selectedProductInfo.files) {
        var fileAlreadyUploaded =
            cp_product_meta.productFiles.local !== undefined
            && cp_product_meta.productFiles.local.hasOwnProperty(file.type)
            && cp_product_meta.productFiles.local[file.type].url !== undefined;

        var fileUploadedHtml = '';
        if (fileAlreadyUploaded) {
            var md5Html = ''

            if (cp_product_meta.productFiles.local[file.type].md5 !== undefined) {
                md5Html = '<br><small>MD5: ' + cp_product_meta.productFiles.local[file.type].md5 + '</small>'
            }

            fileUploadedHtml = ' ' +
                '<span class="file_uploaded">' +
                '   <a href="' + cp_product_meta.productFiles.local[file.type].url + '" target="_blank">' +
                        cp_product_meta.productFiles.local[file.type].name +
                '   </a>' +
                '<span class="cp_remove_file dashicons dashicons-trash"></span>' + md5Html +
                '</span>';
        }

        jQuery('#local-storage-file').append(' ' +
            '<p class="form-field ' + file.type + '_file">' +
            '   <label for="cp_file_upload_product">' +
            file.type[0].toUpperCase() + file.type.substring(1) +
            ' file (' + file.format +')</label>' +
            '   <span class="file_not_uploaded" style="' + (fileAlreadyUploaded ? 'display: none;' : '')  + '">' +
            '       <input type="file" name="cp_file_upload[' + file.type + ']" id="cp_file_upload_' + file.type + '" accept="application/' + file.format + '"/>' +
            '   </span>' + fileUploadedHtml +
            '</p>'
        )
    }
}

function handleFileMethod() {
    if (!selectedProductInfo) {
        return false;
    }

    if (jQuery('#cp_file_method:checked').val() !== 'dynamic') {
        jQuery('#dynamic_file').hide()
    } else {
        jQuery('#dynamic_file').show()
    }

    if (jQuery('#cp_file_method:checked').val() !== 'designer') {
        jQuery('#product_designer').hide()
    } else {
        jQuery('#product_designer').show()
    }

    if (jQuery('#cp_file_method:checked').val() !== 'static') {
        jQuery('.cp_file').hide()
    } else {
        fileStorageChanged();
        jQuery('#cp-storage-type').show();
    }
}

function handleProductDesignerProjects(productDesignerAlias) {
    jQuery('#cp_product_designer_project').html("");

    if (!productDesignerAlias) {
        jQuery('.product-designer-field').hide();
        return false;
    }

    jQuery.ajax({
        url: "/wp-json/cloudprinter/v1/product-designer/projects",
        method: 'POST',
        data: {
            'product_designer_alias': productDesignerAlias,
        },
        headers: {'X-WP-Nonce': cp.nonce},
        dataType: "json",
        success: function (response) {
            jQuery('.product-designer-field').show();
            if (response !== false) {
                jQuery('#cp_product_designer_project_field').html('<select id="cp_product_designer_project" style="width: 300px; margin-bottom: 10px;" name="cp_product_designer_project"></select>');
                jQuery('#cp_product_designer_project').append(' <option value="">Use the empty template</option>')
                for (var project of response) {
                    var selected = '';
                    if (cp_product_meta.productDesignerProject == project.id) {
                        selected = 'selected';
                    }

                    jQuery('#cp_product_designer_project').append('<option value="' + project.id + '" ' + selected + '>' + project.name + '</option>')
                }
            } else {
                jQuery('#cp_product_designer_project_field').html('<input id="cp_product_designer_project" type="text" name="cp_product_designer_project">');
                jQuery('#cp_product_designer_project').val(cp_product_meta.productDesignerProject)
            }
        }
    });
}

function fileStorageChanged() {
    if (jQuery('.cp_file_storage:checked').val() == 'local') {
        jQuery('#local-storage-file').show();
        jQuery('#external-storage-file').hide();
    } else {
        jQuery('#local-storage-file').hide();
        jQuery('#external-storage-file').show();
    }
}

function productOptionTypeChanged(isChangeEvent) {
    if (jQuery('.cp_product_options_type:checked').val() == 'fixed') {
        jQuery('#cp-product-options-fixed').show();
        jQuery('#cp-product-options-dynamic').hide();
        if (isChangeEvent && jQuery('#cp-product-options-fixed').text() === '') {
            prepareProductOptions();
        }
    } else {
        jQuery('#cp-product-options-fixed').hide();
        jQuery('#cp-product-options-dynamic').show();
    }
}

function setUserSelectedDunamicOptions() {
    if (Object.keys(cp_product_meta.product_options_dynamic).length !== 0) {
        for (var optionReference in cp_product_meta.product_options_dynamic) {
            jQuery('#cp_dynamic_product_option_name_' + optionReference).val(cp_product_meta.product_options_dynamic[optionReference].option_name);
            var optionValues = cp_product_meta.product_options_dynamic[optionReference].option_values;
            for (var optionValueReference in optionValues) {
                jQuery('#cp_dynamic_product_option_value_name_' + optionValueReference).val(optionValues[optionValueReference]);
                jQuery('#cp_dynamic_product_option_price_' + optionValueReference).val(cp_product_meta.product_options_dynamic[optionReference].option_prices[optionValueReference]);
                jQuery('#cp_dynamic_product_option_value_' + optionValueReference).prop('checked', true);
            }
        }
    } else {
        jQuery('.dynamic-product_option-value').prop('checked', true);
    }
}
