<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinter\CloudApps\Client\CloudAppsClient;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;

/**
 * Class OrderCancelController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class OrderCancelController
{
    public function __construct()
    {
        add_action( 'woocommerce_order_status_cancell_request_to_cancelled', [$this, 'orderCancel'], 10, 1 );
        add_action( 'woocommerce_order_status_processing_to_cancelled', [$this, 'orderCancel'], 10, 1 );
        add_action( 'woocommerce_order_status_completed_to_cancelled', [$this, 'orderCancel'], 10, 1 );
        add_action( 'woocommerce_order_status_on-hold_to_cancelled', [$this, 'orderCancel'], 10, 1 );

        // Hide cancel status if order is not cancelable.
        add_filter('wc_order_statuses', [$this, 'orderStatuses']);
    }

    public function orderStatuses($statuses)
    {
        $orderId = get_the_ID();
        $action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_STRING);
        if ('shop_order' === get_post_type() && $orderId && isset($action) && 'edit' === $action) {
            if (!$this->orderIsCancelable($orderId)) {
                unset($statuses['wc-cancelled']);
            }
        }

        return $statuses;
    }

    public function orderCancel($orderId)
    {
        $order = new  \WC_Order($orderId);
        if (!$this->orderIsCancelable($orderId)) {
            $order->add_order_note('The order can not be canceled on cloudprinter.com.');
            return false;
        }

        update_post_meta($orderId, '_cp_order_canceled_date', date('Y-m-d H:i:s'));

        $apiKey = get_option('cp_api_key');
        $orderReference = get_post_meta($orderId, '_cp_order_reference', true);
        if ($orderReference) {
            $cloudPrinterClient = new CloudAppsClient($apiKey);
            $response = $cloudPrinterClient->order->cancel($orderReference);

            // Write log
            $message = 'orders/cancel | Request: '. $orderReference . PHP_EOL . 'Response' . json_encode($response);
            LogHelper::debug('debug-api', $message);

            if ($response->getStatusCode() == 200) {
                update_post_meta($orderId, '_cp_status', 'canceled');
            }
        }

        $dynamicProductService = new DynamicProductService();
        $dynamicProductService->deleteFilesForOrder($order);
    }


    private function orderIsCancelable($orderId)
    {
        $orderReference = get_post_meta($orderId, '_cp_order_reference', true);

        if (!$orderReference) {
            return  false;
        }

        $orderInfoData = get_transient('order-info-' . $orderId);

        if (!$orderInfoData) {
            $accessToken = get_option('cp_api_key');
            $cloudPrinterClient = new CloudAppsClient($accessToken);

            $orderInfoResponse = $cloudPrinterClient->order->getInfo($orderReference);
            $orderInfoData = $orderInfoResponse->getData();
            set_transient('order-info-' . $orderId, json_encode($orderInfoData), 180);

        } else {
            $orderInfoData = json_decode($orderInfoData, true);
        }

        if (!$orderInfoData['is_cancelable']) {
            return false;
        }

        return  true;
    }
}
